from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class User(models.Model):

    created_at = models.DateTimeField(verbose_name=_('created at'), null=False, blank=False, auto_now_add=True)
    email = models.EmailField(verbose_name=_('email'), null=False, blank=False, unique=True)
    contract = models.FileField(_('file'), null=True, blank=True, upload_to='documents/')
    is_superuser = models.BooleanField(_('is superuser'), default=True)
    first_name = models.CharField(_('first name'), null=True, blank=True, max_length=100)
    last_name = models.CharField(_('last name'), null=True, blank=True, max_length=100)

    def test(self):
        return 'test'

    def __str__(self):
        return 'user: %s' % self.email


@python_2_unicode_compatible
class Issue(models.Model):

    created_at = models.DateTimeField(verbose_name=_('created at'), null=False, blank=False, auto_now_add=True)
    name = models.CharField(verbose_name=_('name'), max_length=100, null=False, blank=False)
    watched_by = models.ManyToManyField('app.User', verbose_name=_('watched by'), blank=True,
                                        related_name='watched_issues')
    created_by = models.ForeignKey('app.User', verbose_name=_('created by'), null=False, blank=False,
                                   related_name='created_issues')
    solver = models.OneToOneField('app.User', verbose_name=_('solver'), null=True, blank=True,
                                  related_name='solving_issue')
    leader = models.OneToOneField('app.User', verbose_name=_('leader'), null=False, blank=False,
                                  related_name='leading_issue')
    description = models.TextField(verbose_name=_('description'), null=True, blank=True)

    def __str__(self):
        return 'issue: %s' % self.name



from distutils.version import StrictVersion

import django
from django.conf.urls import url, include

from rest_framework.routers import DefaultRouter
from app.resources import UserViewSet, IssueViewSet

# Create a router and register our viewsets with it.
router = DefaultRouter()
router.register(r'users', UserViewSet)
router.register(r'issue', IssueViewSet)

# The API URLs are now determined automatically by the router.
# Additionally, we include the login URLs for the browsable API.
urlpatterns = [
    url(r'^', include(router.urls)),
]

if StrictVersion(django.get_version()) < StrictVersion('1.9'):
    from django.conf.urls import patterns

    urlpatterns = patterns('', *urlpatterns)



import json
import xml.dom.minidom

from collections import OrderedDict

from germanium.tools import assert_true, assert_equal

from unittest.case import TestCase

from app.models import User

from pyston.serializer import serialize


class DirectSerializationTestCase(TestCase):

    def test_serialization(self):
        for i in range(10):
            User.objects.create(is_superuser=True, email='test{}@test.cz'.format(i))
        assert_true(isinstance(json.loads((serialize(User.objects.all()))), list))
        assert_true(isinstance(json.loads((serialize(User.objects.first()))), dict))

        assert_equal(
            set(json.loads((serialize(User.objects.first()))).keys()),
            {'id', 'created_at', '_obj_name', 'email', 'contract', 'solving_issue', 'first_name', 'last_name',
             'manual_created_date'}
        )
        assert_equal(
            set(json.loads((serialize(User.objects.first(), ('id',)))).keys()),
            {'id'}
        )

        xml.dom.minidom.parseString(serialize(User.objects.first(), converter_name='xml'))

    def test_direct_serialization_to_csv_should_create_columns_according_to_required_fieldset(self):
        data = OrderedDict((('a', 1), ('b', 2)))
        assert_equal(
            serialize(data, converter_name='csv'),
            '\ufeff"a: 1\nb: 2"\r\n'
        )
        assert_equal(
            serialize(data, converter_name='csv', requested_fieldset=('a',)),
            '\ufeff"A"\r\n"1"\r\n'
        )
        assert_equal(
            serialize(data, converter_name='csv', requested_fieldset=('a', 'b')),
            '\ufeff"A";"B"\r\n"1";"2"\r\n'
        )
        assert_equal(
            serialize(data, converter_name='csv', requested_fieldset=('a', 'b', 'c')),
            '\ufeff"A";"B";"C"\r\n"1";"2";""\r\n'
        )



from rest_framework import serializers
from rest_framework import viewsets
from rest_framework_recursive.fields import RecursiveField


from .models import User, Issue


class UserSerializer(serializers.ModelSerializer):

    solving_issue = RecursiveField('IssueSerializer')
    watched_issues = serializers.ManyRelatedField(child_relation=RecursiveField('IssueSerializer'))

    class Meta:
        model = User
        fields = ('id', 'created_at', 'email', 'contract', 'first_name', 'last_name', 'is_superuser', 'test',
                  'solving_issue', 'watched_issues')



class UserViewSet(viewsets.ModelViewSet):

    queryset = User.objects.all()
    serializer_class = UserSerializer


class IssueSerializer(serializers.ModelSerializer):

    solver = RecursiveField('UserSerializer')
    #watched_by = RecursiveField('UserSerializer', many=True)

    class Meta:
        model = Issue
        fields = ('id', 'created_at', 'name', 'created_by', 'solver', 'leader', 'watched_by')


class IssueViewSet(viewsets.ModelViewSet):

    queryset = Issue.objects.all()
    serializer_class = IssueSerializer